#include "util.h"
#include "seq.h"
#include "maf.h"
#include "tba_util.h"

static char dna_compl[256] = 
  "                                             -                  "
  " TVGH  CD  M KN   YSA BWXR       tvgh  cd  m kn   ysa bwxr      "
  "                                                                "
  "                                                                ";
/* .............................................-.................. */
/* @ABCDEFGHIJKLMNOPQRSTUVWXYZ[\]^_`abcdefghijklmnopqrstuvwxyz{|}~. */
/* ................................................................ */
/* ................................................................ */

int parse_fasta(char *line, char **ident, int *start, int *end, int *srcSize) {
	char *s, *copy = copy_string(line);
	int ret;

	if (copy[0] != '>' || (s = strchr(copy, ':')) == NULL ||
	     sscanf(s+1, "%d-%d of %d", start, end, srcSize) != 3)
		ret = 0;
	else {
		*s = '\0';
		if (strchr(copy, ' ') != NULL)
			ret = 0;
		else {
			ret = 1;
			*ident = copy_string(copy+1);
		}
	}
	free(copy);
	return ret;
}

void do_revcompl(char *s, int len)
{
  char c, *p = s + len - 1;

  while (s<=p) {
    c = dna_compl[(uchar)*s]; 
    *s = dna_compl[(uchar)*p]; 
    *p = c;
    ++s, --p;
  }
}

// replace every component by its reverse complement
void rc(struct mafAli *a) {
        struct mafComp *c;

        for (c = a->components; c != NULL; c = c->next) {
                c->start = c->srcSize - (c->start + c->size);
                c->strand = (c->strand == '-' ? '+' : '-');
                do_revcompl(c->text, a->textSize);
        }
}

char *seq_ident(char *filename) {
	FILE *fp = ckopen(filename, "r");
	char buf[500], *ident, *retur;
	int x, y, z;

	if (fp == NULL || fgets(buf, 500, fp) == NULL ||
	    parse_fasta(buf, &ident, &x, &y, &z) == 0)
		retur = filename;
	else
		retur = ident;
	fclose(fp);
	return retur;
}

static int compar(const void *a, const void *b)
{
	return (*((struct mafAli **)a))->components->start -
	       (*((struct mafAli **)b))->components->start;
}

struct mafAli **mafArray(struct mafAli *a, int *nali) {
	struct mafAli *b, **A;
	int n, i;

	for (n = 0, b = a; b != NULL; b = b->next)
		++n;
	*nali = n;
	A = ckalloc(n*sizeof(struct mafAli *));
	for (b = a, i = 0; i < n; ++i, b = b->next) {
		if (b == NULL)
			fatal("died in mafArray");
		A[i] = b;
	}
	if (b != NULL)
		fatal("did not reach end in mafArray");
	/* sort alignments by start of first component */
	qsort((void *)A, n, sizeof(struct mafAli *), compar);

for (i = 1; i < n; ++i)
if (A[i-1]->components->start > A[i]->components->start)
fatal("failure in mafArray");
	
	return A;
}

void order_rows(struct mafAli *a, struct mafComp **location, char **species,
  int nspecies) {
	struct mafComp *c, *last;
	int i;

	for (i = 0; i < nspecies; ++i)
		location[i] = NULL;
	for (c = a->components; c != NULL; c = c->next) {
		for (i = 0; i < nspecies; ++i)
			if (same_string(c->src, species[i]))
				break;
		if (i == nspecies)
			fatalf("unnamed species `%s'", c->src);
		if (location[i] != NULL) {
			mafWrite(stderr, a);
			fatalf("species `%s'named more than once",
			  c->src);
		}
		location[i] = c;
	}

	for (last = NULL, i = 0; i < nspecies; ++i)
		if (location[i] != NULL) {
			if (last == NULL)
				a->components = location[i];
			else
				last->next = location[i];
			last = location[i];
		}
	if (last == NULL)
		fatal("maf_order: cannot happen");
	last->next = NULL;
}
